package com.codegeist.fisheye.internal.service;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.codegeist.fisheye.service.FisheyeException;

// check root element.  if 'error', read and throw an error
// API access is disabled
// authentication failed
public final class RestHandler extends DefaultHandler {
	protected StringBuilder buffer = new StringBuilder();
	private int depth = 0;
	private boolean hasError;
	private ContentHandler delegate;
	
	public RestHandler(ContentHandler delegate) {
		this.delegate = delegate;
	}

	@Override
	public void startDocument() throws SAXException {
		depth = 0;
	}
	
	@Override
	public void endDocument() throws SAXException {
		// TODO check depth is 0 again
		hasError = false;
	}
	
	@Override
	public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
		buffer.setLength(0);
		if (depth == 0) {
			if ("error".equals(localName)) {
				hasError = true;
				
			} else if ("response".equals(localName)) {
				hasError = false;
			}
		}
		
		if (depth == 0 && !hasError) {
			delegate.startDocument();
		}
		
		// do not include the first "response" element
		if (depth > 0 && !hasError) {
			delegate.startElement(uri, localName, qName, atts);
		}
		
		depth++;
	}
	
	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException {
		depth--;
		
		// Stop processing and return error to caller
		if (depth == 0 && hasError) {
			String message = buffer.toString();
			FisheyeException.ExceptionType exceptionType = FisheyeException.ExceptionType.SERVER_ERROR;
			if (message.equals("authentication failed")) {
				exceptionType = FisheyeException.ExceptionType.AUTHENTICATION_FAILED;
			} else if (message.equals("API access is disabled")) {
				exceptionType = FisheyeException.ExceptionType.INTERFACE_DISABLED;
			}
			
			throw new SAXException(new FisheyeException(exceptionType, message));
		}
		
		// Do not include the last "response" element
		if (depth > 0 && !hasError) {
			delegate.endElement(uri, localName, qName);
		}
		
		if (depth == 0 && !hasError) {
			delegate.endDocument();
		}
	}
	
	@Override
	public void characters(char ch[], int start, int length) throws SAXException {
		if (hasError) {
			buffer.append(ch, start, length);
		} else if (!hasError && depth > 0){
			delegate.characters(ch, start, length);
		}
	}
}
