package com.codegeist.fisheye.internal.ui.connection;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.team.ui.TeamUI;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.ViewPart;

import com.codegeist.fisheye.core.connection.FisheyeConnection;
import com.codegeist.fisheye.core.connection.IConnectionListener;
import com.codegeist.fisheye.core.connection.IFisheyeConnectionManager;
import com.codegeist.fisheye.domain.PathInfo;
import com.codegeist.fisheye.internal.ui.connection.model.PathModel;
import com.codegeist.fisheye.internal.ui.connection.model.RepositoryModel;
import com.codegeist.fisheye.internal.ui.history.FisheyeHistoryPageSource;
import com.codegeist.fisheye.service.FisheyeException;
import com.codegeist.fisheye.ui.Activator;

public class FisheyeConnectionView extends ViewPart {

	private TreeViewer viewer;
	private List<FisheyeConnection> connections;
	
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL);
		IFisheyeConnectionManager connectionManager = com.codegeist.fisheye.core.Activator.getDefault().getConnectionManager();
		IConnectionListener connectionListener = new IConnectionListener() {
		
			public void connectionRemoved(FisheyeConnection connection) {
				connections.remove(connection);
				viewer.remove(connections, new Object[] {connection});
			}
		
			public void connectionAdded(FisheyeConnection connection) {
				connections.add(connection);
				viewer.add(connections, connection);
			}
		};
		
		connectionManager.addConnectionListener(connectionListener);
		connections = new ArrayList<FisheyeConnection>(connectionManager.getConnections());
		
		viewer.addOpenListener(new IOpenListener() {
		
			public void open(OpenEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				Object selectedObject = selection.getFirstElement();
				if (selectedObject != null) {
					TeamUI.showHistoryFor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), selectedObject, new FisheyeHistoryPageSource());
				}
			}
		});
		
		viewer.setContentProvider(new ITreeContentProvider() {
		
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		
			public void dispose() {
			}
		
			public Object[] getElements(Object inputElement) {
				@SuppressWarnings("unchecked")
				List<FisheyeConnection> connections = (List<FisheyeConnection>) inputElement;
				
				return connections.toArray();
			}
		
			public boolean hasChildren(Object element) {
				if (element instanceof PathModel) {
					PathModel path = (PathModel) element;
					return path.getPathInfo().isDirectory();
				}
				return true;
			}
		
			public Object getParent(Object element) {
				if (element instanceof FisheyeConnection) {
					return null;
				} else if (element instanceof RepositoryModel) {
					return ((RepositoryModel) element).getConnection();
				} else if (element instanceof PathModel) {
					return ((PathModel) element).getParent();
				}
				return null;
			}
		
			public Object[] getChildren(Object parentElement) {
				if (parentElement instanceof FisheyeConnection) {
					FisheyeConnection connection = (FisheyeConnection) parentElement;
					try {
						String[] repositoryNames = connection.getService().listRepositories(null);
						RepositoryModel[] repositories = new RepositoryModel[repositoryNames.length];
						for (int i = 0; i < repositoryNames.length; i++) {
							repositories[i] = new RepositoryModel(connection, repositoryNames[i]);
						}
						return repositories;
					} catch (FisheyeException e) {
						// TODO fix this
						e.printStackTrace();
					}
				} else if (parentElement instanceof RepositoryModel) {
					RepositoryModel repo = (RepositoryModel) parentElement;
					
					try {
						PathInfo[] pathInfos = repo.getConnection().getService().listPaths(null, repo.getName(), "/");
						PathModel[] paths = new PathModel[pathInfos.length];
						for (int i = 0; i < pathInfos.length ; i++) {
							paths[i] = new PathModel(repo, null, pathInfos[i]);
						}
						return paths;
					} catch (FisheyeException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} else if (parentElement instanceof PathModel) {
					PathModel path = (PathModel) parentElement;
					try {
						if (path.getPathInfo().isDirectory()) {
							PathInfo[] pathInfos = path.getRepo().getConnection().getService().listPaths(null, path.getRepo().getName(), path.getPathInfo().getAbsolutePath());
							PathModel[] paths = new PathModel[pathInfos.length];
							for (int i = 0; i < pathInfos.length ; i++) {
								paths[i] = new PathModel(path.getRepo(), path, pathInfos[i]);
							}
							return paths;
						}
					} catch (FisheyeException e) {
						// TODO fix this
						e.printStackTrace();
					}
				}
				
				// TODO Auto-generated method stub
				return null;
			}
		});
		
		ILabelProvider labelProvider = new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof FisheyeConnection) {
					FisheyeConnection connection = (FisheyeConnection)element;
					return connection.getName() + " - " + connection.getConnectionUrl();
				} else if (element instanceof RepositoryModel) {
					RepositoryModel repo = (RepositoryModel) element;
					
					return repo.getName();
				} else if (element instanceof PathModel) {
					PathModel path = (PathModel) element;
					
					return path.getPathInfo().getName();
				}
				
				return "";
			}
		};
		
		viewer.setLabelProvider(labelProvider);
		viewer.setInput(connections);
		
		getViewSite().setSelectionProvider(viewer);
		
		setupActions();
	}

	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	private void setupActions() {
		IActionBars actionBars = getViewSite().getActionBars();
		
		IToolBarManager toolBarManager = actionBars.getToolBarManager();
		
		IAction newConnection = new Action("New Connection") {
			@Override
			public void runWithEvent(Event event) {
				try {
					IHandlerService service = (IHandlerService) getSite().getService(IHandlerService.class);
					service.executeCommand("com.codegeist.fisheye.ui.CreateConnection", event);
				} catch (Exception e) {
					Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Error opening new connection wizard", e));
				}
			}
		};

		toolBarManager.add(newConnection);
		// TODO activate a handler for the delete action
	}
}
